use leptos::*;
use leptos_use::{core::ConnectionReadyState, use_websocket, UseWebsocketReturn};
use serde::Serialize;
use serde_json::json;
use std::rc::Rc;

#[derive(Serialize)]
struct SurrealRequest {
    id: u32,
    method: String,
    params: Vec<SurrealParams>,
}

#[derive(Serialize)]
#[serde(untagged)]
enum SurrealParams {
    Participant,
    SigninParam(SigninParam),
    String(String),
}

#[derive(Serialize)]
struct SigninParam {
    user: String,
    pass: String,
}

#[derive(Clone, Debug)]
pub enum HourKind {
    A,
    B,
    C,
    D,
    Z,
}

#[derive(Clone, Debug)]
pub struct GroupKind {
    hour: HourKind,
    lane: u8,
}

#[derive(Clone, Debug)]
pub struct Participant {
    name: String,
    group: GroupKind,
}

#[derive(Clone, Debug)]
pub struct ParticipantsContext(pub ReadSignal<Vec<Participant>>);

#[derive(Clone)]
pub struct SurrealContext {
    pub message: Signal<Option<String>>,
    send: Rc<dyn Fn(&str)>,
    pub ready_state: Signal<ConnectionReadyState>,
}

impl SurrealContext {
    pub fn new(
        message: Signal<Option<String>>,
        send: Rc<dyn Fn(&str)>,
        ready_state: Signal<ConnectionReadyState>,
    ) -> Self {
        Self {
            message,
            send,
            ready_state,
        }
    }

    #[inline(always)]
    pub fn send(&self, message: &str) {
        (self.send)(message)
    }

    pub fn signin(&self) {
        let request = SurrealRequest {
            id: 1,
            method: String::from("signin"),
            params: vec![SurrealParams::SigninParam(SigninParam {
                user: String::from("root"),
                pass: String::from("root"),
            })],
        };

        self.send(&json!(request).to_string());

        let request = SurrealRequest {
            id: 1,
            method: String::from("use"),
            params: vec![
                SurrealParams::String(String::from("wrb")),
                SurrealParams::String(String::from("timings")),
            ],
        };

        self.send(&json!(request).to_string())
    }
}

pub fn init_surrealdb() {
    let UseWebsocketReturn {
        ready_state,
        message,
        send,
        ..
    } = use_websocket("ws://localhost:80/rpc");

    let (participants, _set_participants) = create_signal::<Vec<Participant>>(vec![Participant {
        name: "User1".to_string(),
        group: GroupKind {
            hour: HourKind::A,
            lane: 1,
        },
    }]);

    provide_context(SurrealContext::new(
        message,
        Rc::new(send.clone()),
        ready_state,
    ));
    provide_context(ParticipantsContext(participants));
}
